@@ -15,18 +15,20 @@ limitations under the License.
 """
 
 import functools
+import logging
 from dataclasses import dataclass
 from types import SimpleNamespace
 from typing import Any, Dict, List, Optional, Tuple
 
 import torch
+from torch.utils.cpp_extension import _get_cuda_arch_flags
 import torch.distributed as dist
 from torch.distributed import ProcessGroup
 
 from ..jit import JitSpec
 from ..jit import env as jit_env
 from ..jit import gen_jit_spec, sm100a_nvcc_flags
-from ..utils import register_custom_op
+from ..utils import register_custom_op, round_up, version_at_least
 from .cuda_ipc import create_shared_buffer, cudart, free_shared_buffer
 
 
@@ -95,6 +97,10 @@ class FP4QuantizationSFLayout:
 
 
 def gen_trtllm_comm_module() -> JitSpec:
+    gencode_flags = _get_cuda_arch_flags()
+    has_sm100 = any(
+        "compute_100" in flag for flag in gencode_flags
+    ) and version_at_least(torch.version.cuda, "12.8")
     return gen_jit_spec(
         "trtllm_comm",
         [
@@ -102,7 +108,7 @@ def gen_trtllm_comm_module() -> JitSpec:
             jit_env.FLASHINFER_CSRC_DIR / "trtllm_allreduce_fusion.cu",
             jit_env.FLASHINFER_CSRC_DIR / "trtllm_moe_allreduce_fusion.cu",
         ],
-        extra_cuda_cflags=sm100a_nvcc_flags,
+        extra_cuda_cflags=sm100a_nvcc_flags if has_sm100 else [],
     )
 
 
@@ -413,6 +419,13 @@ def trtllm_create_ipc_workspace_for_all_reduce(
     group: Optional[ProcessGroup] = None,
 ) -> List[int]:
     """
+    Parameters:
+    - rank: the rank of the current process.
+    - tp_size: the size of the process group.
+    - max_token_num: the maximum number of tokens in a sequence.
+    - hidden_dim: the dimension of the hidden states.
+    - group: the process group to use.
+
     Note:
     This function is used to create a workspace for all reduce.
     The workspace is a list of IPC handles.
@@ -458,7 +471,7 @@ def trtllm_create_ipc_workspace_for_all_reduce(
         lamport_buffer_size,
     ]:
         # all sizes should be aligned to 1LU << 21 bytes (2MB)
-        aligned_size = ((size + (1 << 21) - 1) >> 21) << 21
+        aligned_size = round_up(size, 1 << 21)
         ipc_handles.append(create_shared_buffer(aligned_size, group))
 
     print(
@@ -495,6 +508,8 @@ def trtllm_destroy_ipc_workspace_for_all_reduce(
 
 BarrierFlagCount = 256
 
+MAX_COMM_SIZE = 2147483647 & ~((1 << 21) - 1)  # MAX_INT32 rounded down to 2MB
+
 
 def trtllm_create_ipc_workspace_for_all_reduce_fusion(
     tp_rank: int,
@@ -505,6 +520,14 @@ def trtllm_create_ipc_workspace_for_all_reduce_fusion(
     group: Optional[ProcessGroup] = None,
 ) -> List[int]:
     """
+    Parameters:
+    - tp_rank: the rank of the current process.
+    - tp_size: the size of the process group.
+    - max_token_num: the maximum number of tokens in a sequence.
+    - hidden_dim: the dimension of the hidden states.
+    - use_fp32_lamport: if True, we will use fp32 datatype in allreduce fusion.
+    - group: the process group to use.
+
     Note:
     We would init 3 IPC buffers for trtllm_custom_all_reduce_fusion.
     They are sized as follows:
@@ -530,6 +553,12 @@ def trtllm_create_ipc_workspace_for_all_reduce_fusion(
         if not use_fp32_lamport
         else tp_size * max_token_num * hidden_dim * 4
     )
+    if lamport_comm_size > MAX_COMM_SIZE:
+        logging.warning(
+            f"warning: lamport_comm_size {lamport_comm_size} is greater than MAX_COMM_SIZE {MAX_COMM_SIZE}, set to MAX_COMM_SIZE"
+        )
+        lamport_comm_size = MAX_COMM_SIZE
+
     lamport_buffer_size = lamport_comm_size * 3
 
     # we should init 3 buffers for all reduce fusion:
@@ -539,7 +568,7 @@ def trtllm_create_ipc_workspace_for_all_reduce_fusion(
     for size in [buffer_size, flag_size, lamport_buffer_size]:
         # todo(review): confirm we need this alignment
         # all sizes should be aligned to 1LU << 21 bytes (2MB)
-        aligned_size = ((size + (1 << 21) - 1) >> 21) << 21
+        aligned_size = round_up(size, 1 << 21)
         ipc_handles.append(create_shared_buffer(aligned_size, group))
 
     print(
@@ -547,13 +576,14 @@ def trtllm_create_ipc_workspace_for_all_reduce_fusion(
     )
 
     # Initialize lamport buffer
+    aligned_lamport_buffer_size = round_up(lamport_buffer_size, 1 << 21)
     if use_fp32_lamport:
         trtllm_lamport_initialize(
-            ipc_handles[2][tp_rank], lamport_buffer_size // 4, torch.float32
+            ipc_handles[2][tp_rank], aligned_lamport_buffer_size // 4, torch.float32
         )
     else:
         trtllm_lamport_initialize(
-            ipc_handles[2][tp_rank], lamport_buffer_size // 2, torch.float16
+            ipc_handles[2][tp_rank], aligned_lamport_buffer_size // 2, torch.float16
         )
 
     # initialize workspace
@@ -601,6 +631,10 @@ def trtllm_destroy_ipc_workspace_for_all_reduce_fusion(
     workspace: List[int], group: Optional[ProcessGroup] = None
 ) -> None:
     """
+    Parameters:
+    - workspace: the workspace to destroy.
+    - group: the process group to use.
+
     Note:
     This function is used to destroy a workspace for all reduce fusion.
     The workspace is a list of IPC handles.
@@ -614,6 +648,14 @@ def trtllm_destroy_ipc_workspace_for_all_reduce_fusion(
 
 # allReduce fused quant utils
 def compute_fp4_swizzled_layout_sf_size(total_row, total_column):
+    """
+    Helper function to compute the padded size of the fp4 swizzled layout.
+
+    Parameters:
+    - total_row: the total number of rows.
+    - total_column: the total number of columns.
+    """
+
     def pad_up(x, y):
         return ((x + y - 1) // y) * y
 
@@ -633,6 +675,17 @@ def trtllm_lamport_initialize_all(
     size: int,
     dtype: torch.dtype,
 ) -> None:
+    """
+    Initialize 3 lamport buffers by negative zero.
+
+    Parameters:
+    - buffer_0_ptr: the pointer to the first buffer.
+    - buffer_1_ptr: the pointer to the second buffer.
+    - buffer_2_ptr: the pointer to the third buffer.
+    - size: the size of the buffer.
+    - dtype: the data type of the buffer.
+    """
+
     get_trtllm_comm_module().trtllm_lamport_initialize_all(
         buffer_0_ptr, buffer_1_ptr, buffer_2_ptr, size, dtype
     )
@@ -662,6 +715,32 @@ def trtllm_custom_all_reduce(
     lamport_peer_comm_buffer_ptrs_1: Optional[torch.Tensor],
     lamport_peer_comm_buffer_ptrs_2: Optional[torch.Tensor],
 ) -> None:
+    """
+    Parameters:
+    - inp: the input tensor. [token_num, hidden_dim]
+    - out: the output tensor. [token_num, hidden_dim]
+    - tp_size: the size of the process group.
+    - tp_rank: the rank of the current process.
+    - token_num: the number of tokens in the sequence.
+    - fusion_op_code: the fusion operation code.
+    - strategy_code: the strategy code.
+    - config_code: the config code.
+    - launch_with_pdl: whether to launch with pdl.
+    - flag_value: the flag value.
+    - peer_comm_buffer_ptrs: the peer communication buffer pointers.
+    - peer_barrier_ptrs_in: the peer barrier pointers in.
+    - peer_barrier_ptrs_out: the peer barrier pointers out.
+    - bias: the bias tensor. [hidden_dim]
+    - residual: the residual tensor. [token_num, hidden_dim]
+    - weight: the weight tensor. [hidden_dim]
+    - weight_pre_residual_norm: the weight pre residual norm tensor. [hidden_dim]
+    - eps: the epsilon value.
+    - intermediate_buffer: the intermediate buffer tensor.
+    - lamport_peer_comm_buffer_ptrs_0: the lamport peer communication buffer pointers 0.
+    - lamport_peer_comm_buffer_ptrs_1: the lamport peer communication buffer pointers 1.
+    - lamport_peer_comm_buffer_ptrs_2: the lamport peer communication buffer pointers 2.
+    """
+
     get_trtllm_comm_module().trtllm_custom_all_reduce(
         inp,
         out,
@@ -711,6 +790,54 @@ def trtllm_allreduce_fusion(
     scale_factor: Optional[float],
     layout_code: Optional[FP4QuantizationSFLayout],
 ) -> None:
+    """
+    Parameters:
+    - allreduce_in: the input tensor. [token_num, hidden_dim]
+    - world_size: the size of the process group.
+    - world_rank: the rank of the current process.
+    - token_num: the number of tokens in the sequence.
+    - hidden_dim: the dimension of the hidden states.
+    - workspace_ptrs: the workspace pointers.
+    - launch_with_pdl: whether to launch with pdl.
+    - use_oneshot: whether to use oneshot.
+    - trigger_completion_at_end: whether to trigger completion at the end.
+    - fp32_acc: whether to use fp32 accumulation.
+    - pattern_code: the pattern code.
+    - allreduce_out: the output tensor. [token_num, hidden_dim]
+    - residual_in: the residual input tensor. [token_num, hidden_dim]
+    - residual_out: the residual output tensor. [token_num, hidden_dim]
+    - norm_out: the norm output tensor. [token_num, hidden_dim]
+    - quant_out: the quant output tensor. [token_num, hidden_dim]
+    - scale_out: the scale output tensor. Initialization referece: tests/test_trtllm_allreduce_fusion.py
+    - rms_gamma: the rms gamma tensor. [hidden_dim]
+    - rms_eps: the rms epsilon value.
+    - scale_factor: the scale factor.
+    - layout_code: the layout code.
+
+    Note:
+    Regarding the `use_oneshot` parameter:
+
+    It should only be enabled when:
+    (1) Force to use the one-shot strategy based on your use case.
+    (2) In min-latency mode, the sequence length is less than the one-shot max token number (currently 128).
+
+    Otherwise, it should be disabled (as False).
+    """
+    if not use_oneshot:
+        assert token_num > world_size, "sequence length should be larger than tp_size"
+
+    required_lamport_comm_size = (
+        token_num * hidden_dim * 2 * world_size
+        if allreduce_in.dtype != torch.float32
+        else token_num * hidden_dim * 4 * world_size
+    )
+
+    if required_lamport_comm_size > MAX_COMM_SIZE and use_oneshot:
+        logging.warning(
+            f"required_lamport_comm_size {required_lamport_comm_size} is greater than MAX_COMM_SIZE {MAX_COMM_SIZE}. Cannot use oneshot in this case."
+        )
+        use_oneshot = False
+
     get_trtllm_comm_module().trtllm_allreduce_fusion(
         allreduce_in=allreduce_in,
         world_size=world_size,
@@ -758,6 +885,38 @@ def trtllm_moe_allreduce_fusion(
     quant_out: Optional[torch.Tensor],
     scale_out: Optional[torch.Tensor],
 ) -> None:
+    """
+    Parameters:
+    - world_size: the size of the process group.
+    - world_rank: the rank of the current process.
+    - token_num: the number of tokens in the sequence.
+    - hidden_dim: the dimension of the hidden states.
+    - workspace_ptrs: the workspace pointers.
+    - launch_with_pdl: whether to launch with pdl.
+    - residual_in: the residual input tensor. [token_num, hidden_dim]
+    - rms_gamma: the rms gamma tensor. [hidden_dim]
+    - rms_eps: the rms epsilon value.
+    - scale_factor: the scale factor.
+    - moe_reduction_device_num_experts: the number of experts.
+    - moe_reduction_scale_input: the scale input tensor. [token_num, hidden_dim]
+    - moe_reduction_active_experts_token_input: the active experts token input tensor. [token_num, hidden_dim]
+    - moe_reduction_token_input: the token input tensor. [token_num, hidden_dim]
+    - layout_code: the layout code.
+    - moe_allreduce_out: the moe allreduce output tensor. [token_num, hidden_dim]
+    - residual_out: the residual output tensor. [token_num, hidden_dim]
+    - norm_out: the norm output tensor. [token_num, hidden_dim]
+    - quant_out: the quant output tensor. [token_num // 4, hidden_dim], fp16/bf16 -> fp4
+    - scale_out: the scale output tensor. Initialization referece: tests/test_trtllm_moe_allreduce_fusion.py
+    """
+
+    required_lamport_comm_size = moe_reduction_token_input.numel() * 2 * world_size
+
+    # Note: only one-shot is supported for moe allreduce fusion.
+    if required_lamport_comm_size > MAX_COMM_SIZE:
+        raise ValueError(
+            f"required_lamport_comm_size {required_lamport_comm_size} is greater than MAX_COMM_SIZE {MAX_COMM_SIZE}. Cannot use oneshot in this case."
+        )
+
     get_trtllm_comm_module().trtllm_moe_allreduce_fusion(
         world_size=world_size,
         world_rank=world_rank,
@@ -797,6 +956,31 @@ def trtllm_moe_finalize_allreduce_fusion(
     shared_expert_output: Optional[torch.Tensor],
     expert_scale_factor: Optional[torch.Tensor],
 ) -> None:
+    """
+    Parameters:
+    - allreduce_in: the input tensor. [token_num, top_k, hidden_dim]
+    - residual_in: the residual input tensor. [token_num, hidden_dim]
+    - norm_weight: the norm weight tensor. [hidden_dim]
+    - expanded_idx_to_permuted_idx: the expanded index to permuted index tensor. [token_num, top_k]
+    - norm_out: the norm output tensor. [token_num, hidden_dim]
+    - residual_out: the residual output tensor. [token_num, hidden_dim]
+    - workspace_ptrs: the workspace pointers.
+    - launch_with_pdl: whether to launch with pdl.
+    - world_rank: the rank of the current process.
+    - world_size: the size of the process group.
+    - eps: the epsilon value.
+    - shared_expert_output: the shared expert output tensor. [token_num, hidden_dim]
+    - expert_scale_factor: the expert scale factor tensor. [token_num, top_k]
+    """
+
+    required_lamport_comm_size = allreduce_in.numel() * 2 * world_size
+
+    # Note: only one-shot is supported for moe allreduce fusion.
+    if required_lamport_comm_size > MAX_COMM_SIZE:
+        raise ValueError(
+            f"required_lamport_comm_size {required_lamport_comm_size} is greater than MAX_COMM_SIZE {MAX_COMM_SIZE}. Cannot use oneshot in this case."
+        )
+
     get_trtllm_comm_module().trtllm_moe_finalize_allreduce_fusion(
         allreduce_in=allreduce_in,
         residual_in=residual_in,
