@@ -21,7 +21,9 @@ from typing import Any, List, Literal, Optional, Tuple, Union, overload
 
 import torch
 
+from .cudnn import cudnn_batch_decode_with_kv_cache
 from .jit import (
+    cudnn_fmha_gen_module,
     gen_batch_decode_mla_module,
     gen_batch_decode_module,
     gen_customize_batch_decode_module,
@@ -31,6 +33,7 @@ from .jit import (
     get_single_decode_uri,
     setup_cubin_loader,
     trtllm_fmha_gen_module,
+    trtllm_mla_gen_module,
 )
 from .page import get_seq_lens
 from .prefill import (
@@ -301,6 +304,14 @@ def get_trtllm_fmha_gen_module():
     return op
 
 
+@functools.cache
+def get_trtllm_mla_gen_module():
+    mod = trtllm_mla_gen_module()
+    op = mod.build_and_load()
+    setup_cubin_loader(mod.get_library_path())
+    return op
+
+
 def single_decode_with_kv_cache_with_jit_module(
     jit_module: Any,
     q: torch.Tensor,
@@ -1685,6 +1696,7 @@ class BatchDecodeMlaWithPagedKVCacheWrapper:
     run_return_lse = functools.partialmethod(run, return_lse=True)
 
 
+# todo(Yingyi): update the params list
 def trtllm_batch_decode_with_kv_cache(
     query: torch.Tensor,
     kv_cache: torch.Tensor,
@@ -1725,3 +1737,146 @@ def trtllm_batch_decode_with_kv_cache(
         v_scale,
     )
     return out
+
+
+def _check_trtllm_gen_mla_shape(
+    query,
+    kv_cache,
+    qk_nope_head_dim,
+    kv_lora_rank,
+    qk_rope_head_dim,
+    page_table,
+    page_size,
+):
+    if query.ndim != 3:
+        raise ValueError(f"Expected query.ndim == 3, got {query.ndim}")
+    if kv_cache.ndim != 4:
+        raise ValueError(f"Expected kv_cache.ndim == 4, got {kv_cache.ndim}")
+    if qk_nope_head_dim != 128:
+        raise ValueError(f"Expected qk_nope_head_dim == 128, got {qk_nope_head_dim}")
+    if kv_lora_rank != 512:
+        raise ValueError(f"Expected kv_lora_rank == 512, got {kv_lora_rank}")
+    if qk_rope_head_dim != 64:
+        raise ValueError(f"Expected qk_rope_head_dim == 64, got {qk_rope_head_dim}")
+
+    B_q, H, D_q = query.shape
+    D_ckv = kv_cache.shape[3]
+    # if H != 128:
+    #     raise ValueError(f"Expected 128 heads for query, got {H}")
+    # todo(Yingyi): should we check num_heads == 128? Is this deepseek only?
+    if D_q != D_ckv or D_q != 576:
+        raise ValueError(
+            f"Expected head dim 576 for query and kv_cache, got {D_q} and {D_ckv}"
+        )
+
+    B_block_table, block_num = page_table.shape
+    block_size = page_size
+    if B_q != B_block_table:
+        raise ValueError(
+            f"Expected batch size {B_q} for query and block_table, got {B_q} and {B_block_table}"
+        )
+    if block_num % (128 / block_size) != 0:
+        raise ValueError(
+            f"Expected block_num % (128 / block_size) == 0, got {block_num=} and {block_size=}"
+        )
+
+
+def trtllm_batch_decode_with_kv_cache_mla(
+    query: torch.Tensor,
+    kv_cache: torch.Tensor,
+    workspace_buffer: torch.Tensor,
+    qk_nope_head_dim: int,
+    kv_lora_rank: int,
+    qk_rope_head_dim: int,
+    block_tables: torch.Tensor,
+    seq_lens: torch.Tensor,
+    block_size: int,
+    max_seq_len: int,
+    scale: Optional[float] = 1.0,
+    out: Optional[torch.Tensor] = None,
+    bmm1_scale: Optional[float] = 1.0,
+    bmm2_scale: Optional[float] = 1.0,  # todo(Yingyi): update to be tensor later
+) -> torch.Tensor:
+    """
+    Parameters:
+    query: [batch_size, num_heads, head_dim_qk], head_dim_qk = qk_nope_head_dim (kv_lora_rank) + qk_rope_head_dim, should be concated q_nope + q_rope
+    kv_cache: [num_pages, page_size, head_dim_ckv + head_dim_kpe], should be concated ckv_cache + kpe_cache
+    workspace_buffer: [num_semaphores, 4], used for multi_block mode
+    qk_nope_head_dim: qk_nope_head_dim, must be 128
+    kv_lora_rank: kv_lora_rank, must be 512
+    qk_rope_head_dim: qk_rope_head_dim, must be 64
+    block_tables: page_table of kv cache, [batch_size, num_pages]
+    seq_lens: query_len
+    block_size: page_size
+    max_seq_len: max sequence length
+    scale: model scale of qk, default is 1.0
+    out: output tensor, if not provided, will be allocated internally
+    bmm1_scale: scale for mla bmm1 output, only for fp8 quantization. Per-tensor scale now, so shape is [1].
+    bmm2_scale: scale for mla bmm2 output, only for fp8 quantization. Per-tensor scale now, so shape is [1].
+
+    Note:
+    For FP8 quantization, we could have those per-tensor scales:
+    - q_scale: dynamic scale for query, shape is [1]
+    - k_scale: loaded scale for kv_cache, shape is [1]
+    - v_scale: loaded scale for kv_cache, shape is [1]
+    We could calculate bmm1_scale and bmm2_scale as:
+    - bmm1_scale = q_scale * k_scale
+    - bmm2_scale = v_scale
+
+    TODO: We might support per-head / per-block / any finer-grained quantization in the future.
+    """
+    run_func = get_trtllm_mla_gen_module().trtllm_paged_attention_mla
+
+    if block_size != 32 and block_size != 64:
+        raise ValueError(f"Supported block_size are 32 and 64, got {block_size}")
+
+    _check_trtllm_gen_mla_shape(
+        query,
+        kv_cache,
+        qk_nope_head_dim,
+        kv_lora_rank,
+        qk_rope_head_dim,
+        block_tables,
+        block_size,
+    )
+
+    # todo(Yingyi): update to be tensor
+    # if bmm1_scale is None:
+    #     bmm1_scale = torch.tensor(1.0, device=query.device, dtype=query.dtype)
+    # if bmm2_scale is None:
+    #     bmm2_scale = torch.tensor(1.0, device=query.device, dtype=query.dtype)
+
+    if out is None:
+        out_shape = query.shape[:-1] + (kv_lora_rank,)
+        out = torch.empty(out_shape, dtype=torch.bfloat16, device=query.device)
+    else:
+        _check_shape_dtype_device(out, query.shape, query.dtype, query.device, "out")
+
+    # NOTE(Yingyi): enable scale factor tensor for finer-grained fp8 quantization in the future
+    # if kv_cache.dtype == torch.float8_e4m3fn and (
+    #     bmm1_scale is None or bmm2_scale is None
+    # ):
+    #     raise ValueError(
+    #         "bmm1_scale and bmm2_scale must be provided for fp8 quantization"
+    #     )
+
+    run_func(
+        out,
+        query,
+        kv_cache,
+        workspace_buffer,
+        scale,
+        block_tables,
+        seq_lens,
+        block_size,
+        max_seq_len,
+        qk_nope_head_dim,
+        kv_lora_rank,
+        qk_rope_head_dim,
+        bmm1_scale,
+        bmm2_scale,
+        None,  # acc_q_len, speculative not supported for now
+        None,  # max_attention_window_size, sliding window not supported for now
+        None,  # cyclic_attention_window_size, cyclic window not supported for now
+    )
+    return out
